#
# Mooshak: managing programming contests on the web		APril 2001
# 
#			Zé Paulo Leal 		
#			zp@ncc.up.pt
#
#-----------------------------------------------------------------------------
# file: Contests.tcl
# 
## The contest directory.
##

package provide Contests 1.0

package require data

namespace eval Contests {

    array set Tip {
	Latin-1->uft-8... "Convert all contest Latin-1 (ISO-8859) files to UTF-8"
	Utf-8->latin-1... "Convert all contest UTF-8 files to Latin-1 (ISO-8859)"
    }
}

Attributes Contests {

    Contest	dirs	Contest

}

## Generates an HTML selector for contest
##  active	(boolean) select only active contests { ready running closed }
##  update 	(boolean) update on change
##  registrable	(boolean) list only if registrable
Operation Contests::selector { contest_ 
    {active 1} {update 1} {registrable 1}
} {
    upvar 3 $contest_ contest

    ${_Self_} list_values_texts values texts $active $registrable
    
    switch [ llength $values ] {
	1	{
	    return <i>[ translate::sentence "No contest available" ]</i>
	}	
	2	 { 
	    set contest [ lindex $values 1 ]
	    return [ format \
		 {<input type="hidden" name="contest" value="%s"><b>%s</b>} \
		 [ lindex $values 1 ] [ lindex  $texts 1 ] ]	    
	}
	default  { 
	    if $update {
		return [ layout::menu contest $values $contest $texts 1 ] 
	    } else {
		return [ layout::menu contest $values $contest $texts 1 {} ] 
	    }
	}
    }

}


Operation Contests::list_values_texts {values_ texts_ 
    {active 1} {registrable 1}
} {
    upvar 3 $values_ values
    upvar 3 $texts_ texts
    
    set values {}
    set texts  {}

    
    lappend values  {}
    lappend texts	{}

    foreach cd [ glob -nocomplain -type d ${_Self_}/* ] {

	if [ catch { set seg [ data::open $cd ] } ] continue

	if { $active && [ $cd status {created concluded} ] } continue
	
	if { $registrable && ! [ $cd registrable ] } continue

	lappend texts [ set ${seg}::Designation ]
	lappend values [ file tail $cd ]
    }
}

## Converts files in contests with Latin-1 encoding to UTF-8 
Operation Contests::convert:Latin-1->UFT-8 ? {

    convert_encoding ${_Self_} "ISO-8859" l1 utf-8
}

## Converts files in contests with UTF-8 encoding to Latin-1
Operation Contests::convert:Utf-8->latin-1 ? {

    convert_encoding ${_Self_} "UTF-8" utf-8 l1
}



## Converts files in $dir (and descendents) with $type $from encoding $to encoding
proc Contests::convert_encoding {dir type from to} {
    variable ::Session::Conf

    set no_errors 1

    if [ catch {
	set fd [ open "| find $dir -name *.tcl -exec file {{}} \;"  r ]
    } msg ] {
	puts <pre>$msg</pre>
	set no_errors 0
    }
    while { [ gets $fd line ] > -1 } {	
	set line [ split $line : ]
	if [ string equal [ string trim [ lindex [ lindex $line 1 ] 0 ] ] $type ] {
	    set file [ lindex $line 0 ]
	    puts $file
	    if [ catch { exec recode ${from}..${to} $file }  msg ] {
		puts <pre>$msg</pre>
		set no_errors 0
	    }
	}
    }
    if [ catch { close $fd  } msg ] {
	puts <pre>$msg</pre>
	set no_errors 0
    }

    if $no_errors {
	layout::window_close_after_waiting
    }
    layout::window_open $Conf(controller)?data+$dir select 0

}