#
# Mooshak: managing programming contests on the web		April 2001
# 
#			Zé Paulo Leal 		
#			zp@ncc.up.pt
#
#-----------------------------------------------------------------------------
# file: Quiz.tcl
# 
## Managing a problem in a problem seta

package provide Quiz 1.0

package require data

namespace eval Quiz {

    variable XSLTPROC	/usr/bin/xsltproc		;# xslt processor

    variable Tip	;# Tips on fields content and commands
    variable Help	;# Help on fields content


    array set Tip {
	Duration	"Duration of quiz for each student - HH:MM"
	Printing	"Print exam sheets?"
	Printer		"Printer queue name"
	Shuffle		"Shuffle groups of questions?"
    }

}

Attributes Quiz {
    Fatal	fatal	{}
    Warning	warning {}        

    Duration	text	{}
    Printing	menu	{yes no}
    Printer	text	{}

    Shuffle	menu	{yes no}

    quizGroup	dirs	QuizGroup
    images	dir	Images

}


Operation Quiz::_update_ {} {
    variable XSLTPROC

    check::reset Fatal Warning

    check::execute Fatal $XSLTPROC xslt-proc
    
    if { ! [ regexp {^\d{1,2}:\d{2}$} $Duration ] } {
	append Fatal "Invalid duration: $Duration\n"
    }

    if { $Printer == "" } {
	set Printer [ print::default_printer ]
    }
    
    return [ check::requires_propagation $Fatal ]

}

## Checks vars and sub directories
Operation Quiz::check {} {
    
    check::dir_start
    # check::vars Name {Timeout {^\d+$} }
    # check::fxs Description Program
    check::sub_dirs
    check::dir_end
}


## Generates an XML quiz sheet, both for an empty sheet solving 
## and the filled sheet for correction.
## $grade is a boolean that is true when the student is submitting the quiz
Operation Quiz::sheet {grade} {
    variable ::Session::Conf

    set contest [ contest::active_path ]
    set seg [ data::open $contest ]
    data::open $contest/groups

    foreach {student} \
	[ $contest/groups identify [ contest::team ] {} {Name} ] {}
    set exam [ set ${seg}::Designation ]
    

    set doc [ doc::document \
		  [ doc::xsl-stylesheet "../../styles/quiz.xsl" ] \
		  [ set quiz [ doc::element quiz ] ] ]

    doc::add_child $quiz [ set header [ doc::element header ] ]

    doc::add_child $header [ doc::element_text designation $exam ]
    doc::add_child $header [ doc::element_text duration $Duration ]
    doc::add_child $header [ doc::element_text student $student ]

    if { $grade } {
	set type answered
    } else {
	# generate groups
	set Conf(groups) 	[ ${_Self_} get_groups ]
	set Conf(questions)	{}
	set Conf(choices)	{}

	if { [ info exists Shuffle ] && ! [ string equal Shuffle no ] } {
	    set Conf(groups) [ ::etc::shuffle $Conf(groups) ]
	}

	set type 		empty
    }

    doc::add_child $header [ doc::element_text type $type ]

    set value 0
    foreach group $Conf(groups) {
	data::open $group
	
	set value [ expr $value + [ $group sheet $quiz $grade ] ]
    }
    doc::add_child $header [ doc::element_text value $value ]

    set xml [ doc::serialize $doc ]

    if $grade {
	${_Self_} submit $xml $value
    }

    return  $xml
}

## Print XML file as HTML after applying XST stylesheet
Operation Quiz::submit {xml value} {
    package require Submission

    variable XSLTPROC
    variable ::Submission::ReportFileName

    set team	[ contest::team ]

    # set dir [ contest::transaction submissions {} $team ]
    # This is a hack: all quiz submissions are assigned problem "-"
    # This helps procedures that require submissions with problem ID
    set dir [ contest::transaction submissions "-" $team ]

    set sub  [ data::new $dir Submission ]
    $dir receive_quiz $value
    data::record $dir

    set html_file ${dir}/$ReportFileName
    set xml_file  ${dir}/quiz.xml
 
    if [ regexp {<\?xml-stylesheet\s(.*?)href="(.*?)"\?>} $xml dec - style ] {

	regsub ${dec} $xml {} xml ;# remove xml-stylesheet 
	set fd [ open $xml_file w ] 
	puts $fd $xml
	close $fd

	regsub {\.\./\.\.} $style public_html style
	exec $XSLTPROC --noout		\
	    --stringparam print yes 	\
	    -o $html_file 		\
	    $style $xml_file

	if { [ string equal $Printing yes ] } {	
	    puts stderr "Printing to $Printer file $html_file"
	    ::print::data_file $html_file $Printer   
	}
    } else {
	error "Cannot find stylesheet declaration"
    }

}


Operation Quiz::get_groups {} {

    set groups {}
    foreach dir [ glob -type d -nocomplain ${_Self_}/* ] {
	if [ string equal [ ::data::class $dir ] QuizGroup ] {
	    lappend groups $dir
	}
    }

    return $groups
}
