#
# Mooshak: managing programming contests on the web		April 2001
# 
#			Zé Paulo Leal 		
#			zp@ncc.up.pt
#
#-----------------------------------------------------------------------------
# file: clipboard.tcl
# 
## Basic clipboard functions - cut copy and past - over directories (and files)
## Paste is only possible in directories with same type (can_paste).
## Name and type can be retrived using procedures with that name.
##
## By default, copy and paste update content window.
##
## TODO: Copy/Paste not working !

package provide clipboard 1.0

namespace eval clipboard {
    variable Clip_dir	clipboard	;# clipboard directory

    namespace export cut		;# cuts selected directory or file
    namespace export copy		;# copies selected directory or file
    namespace export paste		;# pastes selected directory or file
    namespace export can_paste 		;# can paste selected directory/file?
    namespace export name		;# name of dir in clipboard
    namespace export type		;# type of dir in clipboard
}

## Cuts selected directory or file
proc clipboard::cut {path {update 1}} {
    variable Clip_dir

    if [ file isdirectory $path ] {
	# if it is an object let it be properly destroyed
	data::open $path
	data::destroy $path
    }
    file::move $path $Clip_dir
    
    if $update {
	content::show [ file dirname $path ] 1
    }
}

## Copies selected directory or file
proc clipboard::copy {path {update 1}} {
    variable Clip_dir

    # keep the clipboard clean
    foreach content [ glob -nocomplain $Clip_dir/* ] {
	file delete -force -- $content
    }

    file::move $path $Clip_dir 1

    if $update {
	content::show $path 1
    }
}

## Return on clipboard name
proc clipboard::name {} {

    if { [ set dir  [ clip ] ] == {} } {
	return {}
    } else {
	return [ file tail $dir ]
    }

}

## Return on clipboard type
proc clipboard::type {} {
    
    if { [ set dir  [ clip ] ] == {} } {
	return {}
    } else {
	return [ data::class $dir ]
    }
    
}

## Last directory in clipboard 
proc clipboard::clip {} {
    variable Clip_dir

    return [ lindex [ lsort -command clipboard::cmp_fx \
	    [ glob -nocomplain $Clip_dir/*  ] ] 0 ]
}

## Compares files using modification time
proc clipboard::cmp_fx {a b} {
    return [ expr [ file mtime $b ] - [ file mtime $a ] ]
}

## Can paste selected directory/file?
proc clipboard::can_paste {dir} {

    return [ string equal [ data::class [ clip ] ] [ data::class $dir ] ]

}


## Pastes selected directory or file
proc clipboard::paste {dir {update 1}} {
    
    if [ can_paste $dir ] {
	catch { eval file delete -force -- [ glob $dir/* ] }
	catch { eval file delete -force -- [ glob $dir/.\[a-zA-Z\]* ] }
	catch { eval file copy -force -- [ glob  [ clip ]/* ] $dir }
	catch { eval file copy -force -- [ glob [ clip ]/.\[a-zA-Z\]* ] $dir }
    }

    if $update { 
	content::show $dir 1
    }
}
